""" Usefull time-related constants."""

SECONDS_IN_MINUTE = 60
MINUTES_IN_HOUR = 60
HOURS_IN_DAY = 24
SECONDS_IN_HOUR = MINUTES_IN_HOUR * SECONDS_IN_MINUTE
MINUTES_IN_DAY = HOURS_IN_DAY * MINUTES_IN_HOUR
SECONDS_IN_DAY = MINUTES_IN_DAY * SECONDS_IN_MINUTE
SECONDS_IN_COMMON_YEAR = 31536000
SECONDS_IN_LEAP_YEAR = 31622400
DAYS_IN_A_WEEK = 7

WEEKDAYS = [
    'Monday',
    'Tuesday',
    'Wednesday',
    'Thursday',
    'Friday',
    'Saturday',
    'Sunday'
]

WESTERN_WORKDAYS = [
    'Monday',
    'Tuesday',
    'Wednesday',
    'Thursday',
    'Friday'
]

JEWISH_WORKDAYS = [
    'Sunday',
    'Monday',
    'Tuesday',
    'Wednesday',
    'Thursday'
]
